// This file is part of JavaScript switch off
//
// Copyright (C) 2010, 2011 Ivaylo Valkov <ivaylo@e-valkov.org>
//
// JavaScript switch off is free software: you can redistribute it
// and/or modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.
//
// JavaScript switch off is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JavaScript switch off.  If not, see <http://www.gnu.org/licenses/>.
//
// JavaScript switch off version 0.2
// Turn off (or on) JavaScript from statusbar icon in Epiphany.
//
// Home page: http://e-valkov.org/epiphany-extensions/js-switch-off/

Epiphany = imports.gi.Epiphany;
Gdk = imports.gi.Gdk;
GConf = imports.gi.GConf;
GLib = imports.gi.GLib;

var js_enabled_gconf_key = "/apps/epiphany/web/javascript_enabled";

var statusbar_evbox_key = "EphyJsSwitchOffExtensionStatusbarEvbox";

var js_icon_on = GLib.build_filenamev(["/usr", "share",
				       "epiphany-extensions",
				       "js-switch-off",
				       "js-switch-on.png"]);

if (!GLib.file_test(js_icon_on, GLib.FileTest.EXISTS))
{
    js_icon_on = GLib.build_filenamev([GLib.get_home_dir(),
				       ".gnome2", "epiphany",
				       "extensions","data", 
				       "js-switch-off", 
				       "js-switch-on.png"]);
}

var js_icon_off = GLib.build_filenamev(["/usr", "share", 
					"epiphany-extensions",
					"js-switch-off", 
					"js-switch-off.png"]);

if (!GLib.file_test(js_icon_off, GLib.FileTest.EXISTS))
{
    js_icon_off = GLib.build_filenamev([GLib.get_home_dir(), ".gnome2",
					"epiphany", "extensions","data",
					"js-switch-off",
					"js-switch-off.png"]);
}

// Keep track of windows. Used to change all the statusbar icons.
// Event-driven replacement of the icon on GConf value change with
// every event_box, did not work for some reason. All callbacks
// received only one of the window/event_box/icon.
var windows = new Array();

// See attach_window();
var gconf_client =  null;
var gconf_notify_id = null

// Returns an icon representing the JavaScript state (on/off)
// according to the is_js_on argument(0/1; true/false)
var create_js_state_icon = function(is_js_on)
{
    var icon = null;

    if (is_js_on)
    {
	icon = new Gtk.Image.from_file(js_icon_on);
    }
    else
    {
	icon = new Gtk.Image.from_file(js_icon_off);
    }

    return icon;
}

// Set the tooltip of the event box that holds the icon on the
// statusbar according to the is_js_on argument.
var set_event_box_tooltip = function(event_box, is_js_on)
{
    if (is_js_on)
    {
	event_box.set_tooltip_text("Turn off JavaScript");
    }
    else
    {
	event_box.set_tooltip_text("Turn on JavaScript");
    }

    return event_box; 
}

// Called before/when the extension is enabled.  Makes the statusbar
// icon and attaches it.
var create_statusbar_icon = function(window)
{
    var statusbar = window.get_statusbar();
    
    if (!statusbar)
	return;

    event_box = new Gtk.EventBox();
    var is_js_on = gconf_client.get_bool(js_enabled_gconf_key);

    var icon = create_js_state_icon(is_js_on);
    set_event_box_tooltip(event_box ,is_js_on);

    event_box.add(icon);
    event_box.show();
    icon.show();

    statusbar.add_widget(event_box);

    statusbar.set_data(statusbar_evbox_key, event_box);
    var signal= event_box.signal.
	button_press_event.connect(statusbar_icon_clicked_cb);

    statusbar[statusbar_evbox_key] = event_box;
}

// Called before the extension is disabled.  Removes the statusbar
// icon.
var destroy_statusbar_icon = function(window)
{
    var statusbar = window.get_statusbar();

    if (!statusbar)
	return;

    var event_box = statusbar[statusbar_evbox_key];

    statusbar.remove_widget(event_box);
}

// The GConf key for enabled/disabled JavaScript had changed.
var gconf_value_changed_cb  = function(gconf_client, cncx_id, entry)
{
    
    var is_js_on = entry.value.get_bool();

    for (var w=0, l=windows.length; w<l; w++)
    {
	var window = windows[w];
	var statusbar = window.get_statusbar();
	
	var event_box = statusbar[statusbar_evbox_key];
	var icon = event_box.get_child();
    
	if (icon)
	{
	    icon.destroy();
	}
	
	icon = create_js_state_icon(is_js_on);
	set_event_box_tooltip(event_box,is_js_on);
	
	event_box.add(icon);
	event_box.show();
	icon.show();
    }
}

// The statusbar icon is clicked.
var statusbar_icon_clicked_cb = function(event_box, event)
{
    var is_js_on = gconf_client.get_bool(js_enabled_gconf_key);

    // Change the setting to opposite value
    gconf_client.set_bool(js_enabled_gconf_key, !is_js_on);

    return true;
}

// Extension init

var detach_window = function(window)
{  
    windows.pop(window);
    
    // Must be the last window.  Remove the callback for GConf value
    // changes.
    if (windows.length == 0 && gconf_notify_id)
    {
	 gconf_client.notify_remove(gconf_notify_id);
    }

    destroy_statusbar_icon(window);
}

var attach_window = function(window)
{
    if (!gconf_client)
    {
	gconf_client = GConf.Client.get_default();
    }

    // If this is called too early and out of this function Epiphany
    // quits with segmentation fault. This way it is still one instance
    // for the entire extension/
    if (!gconf_notify_id)
    {
	gconf_notify_id = 
	    gconf_client.notify_add(js_enabled_gconf_key,
				    gconf_value_changed_cb);
    }

    create_statusbar_icon(window);
    windows.push(window);
}

extension=
{
    attach_window: attach_window,
    detach_window: detach_window,
}
